import {StoreComponent} from "../../../../app/storage/StoreComponent";
import {Item} from "../game-model/inventory/Item";
import {ItemQuality} from "../game-model/inventory/ItemQuality";
import {ItemProperty} from "../game-model/inventory/ItemProperty";
import {ItemPropertyDescription} from "../game-model/inventory/ItemPropertyDescription";
import {ModelComponentType} from "../game-model/ModelComponentType";

export class ItemParser {

  public static parseItems(items: StoreComponent[]): Item[] {
    const parsedItems: Item[] = []
    items.forEach(item => parsedItems.push(this.parseSingleItem(JSON.parse(item.jsonString))));
    return parsedItems;
  }

  private static parseSingleItem(itemData: any): Item {
    const componentName = itemData.componentName;
    const componentDescription = itemData.componentDescription;

    const perQualityProperties = this.parseQualityPropertyReferences(itemData.perQualityProperties);
    const qualties = this.parseQualities(itemData.possible_qualities, perQualityProperties)
    const itemProperties = this.parseItemProperties(itemData.itemProperties)

    const item = new Item(componentName, componentDescription, ModelComponentType.ITEM);
    item.possible_qualities = qualties;
    item.itemProperties = itemProperties;
    item.perQualityProperties = perQualityProperties;

    return item;
  }

  private static parseItemProperties(propertyData: any): ItemProperty[] {
    const itemProperties : ItemProperty[] = []
    for(let i=0; i<propertyData.length; i++) {
      const propertyName = propertyData[i].propertyName;
      const propertyDescription = propertyData[i].propertyDescription;
      const property = propertyData[i].property;

      itemProperties.push(new ItemProperty(propertyName, propertyDescription, property))
    }
    return itemProperties;
  }

  private static parseQualities(qualityData: any, perQualityProperties: ItemPropertyDescription[]): ItemQuality[]  {
    const qualities: ItemQuality[] = []
    for(let i=0; i<qualityData.length; i++){
      const qualityStep = qualityData[i].quality_step;
      const qualityProperties = this.parseQualityProperty(qualityData[i].quality_propertes, perQualityProperties)

      const itemQuality = new ItemQuality(qualityStep);
      itemQuality.quality_propertes = qualityProperties;
      qualities.push(itemQuality)
    }

    return qualities;
  }

  private static parseQualityProperty(qualityPropertyData: any, propertyDescriptions: ItemPropertyDescription[]): ItemProperty[] {
    const result: ItemProperty[] = []
    for(let i=0; i<qualityPropertyData.length; i++){
      const propertyName = qualityPropertyData[i].propertyName;
      const propertyDescription = this.findMappingPropertyDescription(propertyDescriptions, propertyName)!.propertyDescription
      const property = qualityPropertyData[i].property;

      result.push(new ItemProperty(propertyName, propertyDescription, property))
    }
    return result;
  }

  private static findMappingPropertyDescription(propertyDescriptions: ItemPropertyDescription[], propertyName: string) {
    return propertyDescriptions.find(property => property.propertyName === propertyName);
  }

  private static parseQualityPropertyReferences(qualityPropertyData: any): ItemPropertyDescription[] {
    const result: ItemPropertyDescription[] = []
    for(let i=0; i<qualityPropertyData.length; i++){
      const propertyName = qualityPropertyData[i].propertyName;
      const propertyDescription = qualityPropertyData[i].propertyDescription;
      result.push(new ItemPropertyDescription(propertyName, propertyDescription))
    }
    return result;
  }


}
