import {ScriptAccount} from "../../game-model/scriptAccounts/ScriptAccount";
import {ScriptAccountCondition} from "../../game-model/gamesystems/conditions/ScriptAccountCondition";
import {max, min} from "rxjs";

export class ScriptAccountConditionParser {

  private scriptAccounts: ScriptAccount[]


  constructor(scriptAccounts: ScriptAccount[]) {
    this.scriptAccounts = scriptAccounts;
  }

  parseStoredConditions(conditionData: any): ScriptAccountCondition[] {
    const conditions: ScriptAccountCondition[]  = []
    for(let i=0; i<conditionData.length; i++) {
      const parsedCondition = this.parseSingleCondition(conditionData[i]);
      if(parsedCondition != undefined) {
        conditions.push(parsedCondition)
      }
    }
    return conditions
  }

  private parseSingleCondition(conditionData: any): ScriptAccountCondition | undefined{
    const referencedScriptAccount = this.findScriptAccountByName(conditionData.scriptAccount);
    if(referencedScriptAccount != undefined) {
      const minValue = conditionData.minValue;
      const maxValue = conditionData.maxValue;
      return ScriptAccountCondition.constructScriptAccountCondition(referencedScriptAccount, minValue, maxValue);
    } else {
      return undefined;
    }
  }

  private findScriptAccountByName(scriptAccountName: string) {
    return this.scriptAccounts.find(scriptAccount => scriptAccount.componentName === scriptAccountName);
  }
}
