import {Gamesystem} from "../game-model/gamesystems/Gamesystem";
import {StoreComponent} from "../../../../app/storage/StoreComponent";
import {SimpleGamesystem} from "../game-model/gamesystems/SimpleGamesystem";
import {ProductGamesystem} from "../game-model/gamesystems/ProductGamesystem";
import {SerializeConstants} from "./SerializeConstants";
import {ModelComponentType} from "../game-model/ModelComponentType";
import {ProductTemplateSystem} from "../game-model/templates/productGamesystem/ProductTemplateSystem";

export class GamesystemSerializer {

  private static IGNORED_SIMPLE_ATTRIBUTES  = ["parentGamesystem", 'incomingTransitions', "outgoingTransitions", "unsaved", "type", "conditionMap", "actionMap", "initialMap"]

  public static serializeGamesystems(gamesystems: Gamesystem<any, any>[]): StoreComponent[] {
    let storedGamesystems: StoreComponent[] = []
    gamesystems.forEach(gamesystem => storedGamesystems = storedGamesystems.concat(this.serializeSingleGamesystem(gamesystem)))
    return storedGamesystems
  }

  private static serializeSingleGamesystem(gamesystem: Gamesystem<any, any>): StoreComponent[] {
    if(gamesystem instanceof SimpleGamesystem) {
      console.log("Simple Gamesystem")
      return [this.serializeSimpleGamesystem(gamesystem as SimpleGamesystem)]
    } else {
      return this.serializeProductGamesystem(gamesystem as ProductGamesystem)
    }
  }

  private static serializeSimpleGamesystem(simpleGamesystem: SimpleGamesystem): StoreComponent {
    const fileName = this.computeSimpleGamesystemPath(simpleGamesystem);
    console.log(fileName)
    const jsonString = JSON.stringify(simpleGamesystem, (key, value) => {

      if(this.IGNORED_SIMPLE_ATTRIBUTES.includes(key)) {
        return undefined
      } else if(key === 'startingState' || key === 'endingState') {
        return value.stateLabel
      } else if(key === 'scriptAccount') {
        return value.componentName
      } else {
        return value;
      }
    }, SerializeConstants.JSON_INDENT)

    return new StoreComponent(jsonString, fileName, ModelComponentType.GAMESYTEM)
  }

  private static serializeProductGamesystem(productGamesystem: ProductGamesystem): StoreComponent[] {
    const storedGamesystems: StoreComponent[] = []

    const fileName = this.computeProductGamesystemPath(productGamesystem)
    const innerGamesystemJsonArray: {'componentName': string}[] = []
    productGamesystem.innerGamesystems.forEach(innerGamesystem => {
      innerGamesystemJsonArray.push({
        'componentName': innerGamesystem.componentName
      })

      const storedChildsystems = this.serializeSingleGamesystem(innerGamesystem);
      storedChildsystems.forEach(storedChildsystem => storedGamesystems.push(storedChildsystem))
    })

    let jsonString;
    if(productGamesystem instanceof ProductTemplateSystem) {
      jsonString = {
        'componentName': productGamesystem.componentName,
        'componentDescription': productGamesystem.componentDescription,
        'childsystems': innerGamesystemJsonArray,
        'templateType': productGamesystem.templateType,
        'symmetric': productGamesystem.symmetric,
        'generateIsolatedStates':productGamesystem.generateIsolatedStates
      }
    } else {
      jsonString = {
        'componentName': productGamesystem.componentName,
        'componentDescription': productGamesystem.componentDescription,
        'childsystems': innerGamesystemJsonArray,
        'generateIsolatedStates':productGamesystem.generateIsolatedStates
      }
    }



    const storedProductsystem = new StoreComponent(JSON.stringify(jsonString, null, SerializeConstants.JSON_INDENT), fileName, ModelComponentType.GAMESYTEM)
    storedGamesystems.push(storedProductsystem)

    return storedGamesystems;
  }

  private static computeSimpleGamesystemPath(simpleGamesystem: SimpleGamesystem): string {
    if(simpleGamesystem.parentGamesystem == undefined) {
      return simpleGamesystem.componentName
    } else {
      const pathElements: string[] = [simpleGamesystem.componentName]
      let currentGamesystem: ProductGamesystem | undefined = simpleGamesystem.parentGamesystem
      while(currentGamesystem != undefined) {
        pathElements.unshift(currentGamesystem.componentName)
        currentGamesystem = currentGamesystem.parentGamesystem
      }
      let output = ""
      for(let i=0; i<pathElements.length; i++) {
        output += pathElements[i] + "/"
      }
      return output.slice(0, -1)
    }
  }

  private static computeProductGamesystemPath(productGamesystem: ProductGamesystem): string {
    if(productGamesystem.parentGamesystem == undefined) {
      return productGamesystem.componentName + "/" + productGamesystem.componentName
    } else {
      const pathElements: string[] = [productGamesystem.componentName + "/" + productGamesystem.componentName]
      let currentGamesystem: ProductGamesystem | undefined = productGamesystem.parentGamesystem
      while(currentGamesystem != undefined) {
        pathElements.unshift(currentGamesystem.componentName)
        currentGamesystem = currentGamesystem.parentGamesystem
      }
      let output = ""
      for(let i=0; i<pathElements.length; i++) {
        output += pathElements[i] + "/"
      }
      return output.slice(0, -1)
    }
  }
}
